/*
 * FMDLog
 * Wideband O2 Display/Datalogging Application
 * For the TechEdge DIY-WB series of devices
 * (c) 2002-2003, Jonathan Burchmore
 *
 * Your use of this software is governed by the GNU General Public License, v2.
 * See the file COPYING.txt, or http://www.fsf.org/copyleft/gpl.txt
 *
 * This software is provided with ABSOLUTELY NO WARRANTY.  See the GNU General
 * public license for additional information.
 */

#include <PalmOS.h>
#include "fmdlog.h"

typedef struct _ConfigListItem
{
	void			*data;
	unsigned int	size;
} ConfigListItem;

#define CONFIG_GRAPH( x )	{ &x.display,	sizeof( x.display ) }, \
							{ &x.graph,		sizeof( x.graph ) }, \
							{ &x.log,		sizeof( x.log ) }, \
							{ &x.low,		sizeof( x.low ) }, \
							{ &x.high,		sizeof( x.high ) },
#define CONFIG_ITEM( x )	{ &x, sizeof( x ) },

static ConfigListItem	ConfigList[] =
{
	CONFIG_GRAPH( graph_afr )
	CONFIG_GRAPH( graph_rpm )
	CONFIG_GRAPH( graph_v1 )
	CONFIG_GRAPH( graph_v2 )

	CONFIG_ITEM( rpm_pulses )
	
	CONFIG_ITEM( scaling_v1_points )
	CONFIG_ITEM( scaling_v2_points )

	CONFIG_ITEM( scaling_v1_mv[ 0 ] ) CONFIG_ITEM( scaling_v1_mv[ 1 ] ) CONFIG_ITEM( scaling_v1_mv[ 2 ] ) CONFIG_ITEM( scaling_v1_mv[ 3 ] )
	CONFIG_ITEM( scaling_v1_mv[ 4 ] ) CONFIG_ITEM( scaling_v1_mv[ 5 ] ) CONFIG_ITEM( scaling_v1_mv[ 6 ] ) CONFIG_ITEM( scaling_v1_mv[ 7 ] )
	CONFIG_ITEM( scaling_v1_mv[ 8 ] ) CONFIG_ITEM( scaling_v1_mv[ 9 ] ) CONFIG_ITEM( scaling_v1_mv[ 10 ] )

	CONFIG_ITEM( scaling_v1_out[ 0 ] ) CONFIG_ITEM( scaling_v1_out[ 1 ] ) CONFIG_ITEM( scaling_v1_out[ 2 ] ) CONFIG_ITEM( scaling_v1_out[ 3 ] )
	CONFIG_ITEM( scaling_v1_out[ 4 ] ) CONFIG_ITEM( scaling_v1_out[ 5 ] ) CONFIG_ITEM( scaling_v1_out[ 6 ] ) CONFIG_ITEM( scaling_v1_out[ 7 ] )
	CONFIG_ITEM( scaling_v1_out[ 8 ] ) CONFIG_ITEM( scaling_v1_out[ 9 ] ) CONFIG_ITEM( scaling_v1_out[ 10 ] )

	CONFIG_ITEM( scaling_v2_mv[ 0 ] ) CONFIG_ITEM( scaling_v2_mv[ 1 ] ) CONFIG_ITEM( scaling_v2_mv[ 2 ] ) CONFIG_ITEM( scaling_v2_mv[ 3 ] )
	CONFIG_ITEM( scaling_v2_mv[ 4 ] ) CONFIG_ITEM( scaling_v2_mv[ 5 ] ) CONFIG_ITEM( scaling_v2_mv[ 6 ] ) CONFIG_ITEM( scaling_v2_mv[ 7 ] )
	CONFIG_ITEM( scaling_v2_mv[ 8 ] ) CONFIG_ITEM( scaling_v2_mv[ 9 ] ) CONFIG_ITEM( scaling_v2_mv[ 10 ] )

	CONFIG_ITEM( scaling_v2_out[ 0 ] ) CONFIG_ITEM( scaling_v2_out[ 1 ] ) CONFIG_ITEM( scaling_v2_out[ 2 ] ) CONFIG_ITEM( scaling_v2_out[ 3 ] )
	CONFIG_ITEM( scaling_v2_out[ 4 ] ) CONFIG_ITEM( scaling_v2_out[ 5 ] ) CONFIG_ITEM( scaling_v2_out[ 6 ] ) CONFIG_ITEM( scaling_v2_out[ 7 ] )
	CONFIG_ITEM( scaling_v2_out[ 8 ] ) CONFIG_ITEM( scaling_v2_out[ 9 ] ) CONFIG_ITEM( scaling_v2_out[ 10 ] )

	CONFIG_ITEM( alarm_afr )
	CONFIG_ITEM( alarm_rpm )
	CONFIG_ITEM( alarm_v1 )
	CONFIG_ITEM( alarm_v2 )

	CONFIG_ITEM( alarm_AFR_low )
	CONFIG_ITEM( alarm_AFR_high )
	CONFIG_ITEM( alarm_AFR_Hz )

	CONFIG_ITEM( alarm_RPM_low )
	CONFIG_ITEM( alarm_RPM_high )
	CONFIG_ITEM( alarm_RPM_Hz )

	CONFIG_ITEM( alarm_Vuser1_low )
	CONFIG_ITEM( alarm_Vuser1_high )
	CONFIG_ITEM( alarm_Vuser1_Hz )

	CONFIG_ITEM( alarm_Vuser2_low )
	CONFIG_ITEM( alarm_Vuser2_high )
	CONFIG_ITEM( alarm_Vuser2_Hz )

	CONFIG_ITEM( alarm_min_RPM )

	CONFIG_ITEM( smoothing_afr )
	CONFIG_ITEM( smoothing_rpm )
	CONFIG_ITEM( smoothing_v1 )
	CONFIG_ITEM( smoothing_v2 )

	CONFIG_ITEM( v1_label )
	CONFIG_ITEM( v2_label )
	
	CONFIG_ITEM( color_afr )
	CONFIG_ITEM( color_rpm )
	CONFIG_ITEM( color_v1 )
	CONFIG_ITEM( color_v2 )

	CONFIG_ITEM( lambda )

	CONFIG_ITEM( skip_factor )
	CONFIG_ITEM( rpm_factor )

	CONFIG_ITEM( packet_type )

	{ NULL, 0 }
};

/*
 * Configuration Functions
 */

void Config_Load()
{
	int i;
	UInt32 size, recordpos;
	unsigned char *recordp;
	MemHandle dbrecord;
	DmOpenRef configDB;
	RGBColorType black;

	packet_type							= PF_TEv15;
	packet_length						= 12;
	packet_crc_pos						= 11;

	graph_afr.display					= 1;			/* 1b */
	graph_afr.graph						= 1;			/* 1b */
	graph_afr.log						= 1;			/* 1b */
	graph_afr.low						= 10.0;			/* 1b */
	graph_afr.high						= 20.0;			/* 1b */

	graph_rpm.display					= 1;			/* 1b */
	graph_rpm.graph						= 1;			/* 1b */
	graph_rpm.log						= 1;			/* 1b */
	graph_rpm.low						= 0;			/* 2b */
	graph_rpm.high						= 10000;		/* 2b */

	graph_v1.display = graph_v2.display	= 1;			/* 1b */
	graph_v1.graph = graph_v2.graph		= 1;			/* 1b */
	graph_v1.log = graph_v2.log			= 1;			/* 1b */
	graph_v1.low = graph_v2.low			= 0;			/* 2b */
	graph_v1.high = graph_v2.high		= 5000;			/* 2b */

	rpm_factor							= 12000000;		/* 4b */
	rpm_pulses							= 4;			/* 1b */

	scaling_v1_points					= 0;			/* 1b */
	scaling_v2_points					= 0;			/* 1b */

										/* scaling_v1_mv:  32b */
										/* scaling_v1_out: 32b */
										/* scaling_v2_mv:  32b */
										/* scaling_v2_out: 32b */

	alarm_afr							= 0;			/* 1b */
	alarm_rpm							= 0;			/* 1b */
	alarm_v1							= 0;			/* 1b */
	alarm_v2							= 0;			/* 1b */

	alarm_AFR_low						= 0.00;			/* 1b */
	alarm_AFR_high						= 25.0;			/* 1b */
	alarm_AFR_Hz						= 500;			/* 2b */

	alarm_RPM_low						= 0;			/* 2b */
	alarm_RPM_high						= 0;			/* 2b */
	alarm_RPM_Hz						= 500;			/* 2b */

	alarm_Vuser1_low					= 0;			/* 2b */
	alarm_Vuser1_high					= 0;			/* 2b */
	alarm_Vuser1_Hz						= 500;			/* 2b */

	alarm_Vuser2_low					= 0;			/* 2b */
	alarm_Vuser2_high					= 0;			/* 2b */
	alarm_Vuser2_Hz						= 500;			/* 2b */

	alarm_min_RPM						= 0;			/* 2b */

	smoothing_afr						= 0;			/* 1b */
	smoothing_rpm						= 0;			/* 1b */
	smoothing_v1						= 0;			/* 1b */
	smoothing_v2						= 0;			/* 1b */
	max_smoothing						= 0;

	StrCopy( v1_label, "Vuser1" );						/* 11b */
	StrCopy( v2_label, "Vuser2" );						/* 11b */

	black.r	= 0;
	black.g	= 0;
	black.b	= 0;
	
	color_afr							= WinRGBToIndex( &black );	/* 1b */
	color_rpm							= color_afr;				/* 1b */
	color_v1							= color_afr;				/* 1b */
	color_v2							= color_afr;				/* 1b */

	lambda								= 0;			/* 1b */
	skip_factor							= 0;			/* 1b */

	if ( ( configDB = DmOpenDatabaseByTypeCreator( WBO2_CONFIG_DBTYPE, WBO2_ID, dmModeReadWrite ) ) != NULL )
	{
		if ( ( dbrecord	= DmQueryRecord( configDB, 0 ) ) != NULL )
		{
			i			= 0;
			size		= MemHandleSize( dbrecord );
			recordp		= MemHandleLock( dbrecord );
			recordpos	= 0;

			while ( ConfigList[ i ].size && recordpos + ConfigList[ i ].size <= size )
			{
				MemMove( ConfigList[ i ].data, &recordp[ recordpos ], ConfigList[ i ].size );
				recordpos += ConfigList[ i++ ].size;
			}

			MemPtrUnlock( recordp );
		}

		DmCloseDatabase( configDB );
	}

	switch ( packet_type )
	{
		case PF_TEv1	: packet_length = 1;	packet_crc_pos = 0;		break;
		case PF_TEv15	: packet_length = 12;	packet_crc_pos = 11;	break;
		case PF_TEv20	: packet_length	= 28;	packet_crc_pos = 27;	break;
	}
}

void Config_Save()
{
	int i;
	MemPtr recordp;
	MemHandle dbrecord;
	DmOpenRef configDB;
	UInt16 dbrecord_index;
	UInt32 size, offset;

	if ( ( configDB = DmOpenDatabaseByTypeCreator( WBO2_CONFIG_DBTYPE, WBO2_ID, dmModeReadWrite ) ) == 0 )
	{
		DmCreateDatabase( 0, WBO2_CONFIG_DBNAME, WBO2_ID, WBO2_CONFIG_DBTYPE, false );
		configDB = DmOpenDatabaseByTypeCreator( WBO2_CONFIG_DBTYPE, WBO2_ID, dmModeReadWrite );
	}

	if ( configDB )
	{
		dbrecord_index		= 0;

		if ( DmNumRecords( configDB ) != 0 )
		{
			DmRemoveRecord( configDB, 0 );
		}

		/*
		 * Calculate the size of the config record
		 */

		for ( size = 0, i = 0; ConfigList[ i ].size; i++ )
		{
			size += ConfigList[ i ].size;
		}

		dbrecord	= DmNewRecord( configDB, &dbrecord_index, size );
		recordp		= MemHandleLock( dbrecord );
		offset		= 0;

		for ( i = 0; ConfigList[ i ].size; i++ )
		{
			DmWrite( recordp, offset, ConfigList[ i ].data, ConfigList[ i ].size );
			offset += ConfigList[ i ].size;
		}

		MemPtrUnlock( recordp );
		DmReleaseRecord( configDB, dbrecord_index, 0 );
		DmCloseDatabase( configDB );
	}
}
