/*
 * FMDLog
 * Wideband O2 Display/Datalogging Application
 * For the TechEdge DIY-WB series of devices
 * (c) 2002-2003, Jonathan Burchmore
 *
 * Your use of this software is governed by the GNU General Public License, v2.
 * See the file COPYING.txt, or http://www.fsf.org/copyleft/gpl.txt
 *
 * This software is provided with ABSOLUTELY NO WARRANTY.  See the GNU General
 * public license for additional information.
 */

#include <PalmOS.h>
#include "resource.h"
#include "fmdlog.h"

/*
 * Logging Functions
 */

void Log_Start()
{
	WinDrawChars( "Logging", 7, 159 - FntCharsWidth( "Logging", 7 ), 0 );

	logActive				= 1;
	dbrecord_queue_pos		= 0;
	dbrecord_queue_samples	= 0;
	dbrecord_index			= -1;
	dbrecord_first_index	= -1;
	dbrecord_size			= 0;

	MemSet( &dbrecord_header,	sizeof( WBO2Record_Header ),	0 );
	MemSet( &dbrecord_desc,		sizeof( WBO2Record_Desc ),		0 );

	dbrecord_header.prev	= -1;
	dbrecord_header.next	= -1;

	if ( graph_afr.log )	{ dbrecord_desc.fields |= WBO2_FIELD_VWB; dbrecord_item_size += 2; }
	if ( graph_rpm.log )	{ dbrecord_desc.fields |= WBO2_FIELD_RPM; dbrecord_item_size += 2; }
	if ( graph_v1.log )		{ dbrecord_desc.fields |= WBO2_FIELD_VU1; dbrecord_item_size += 2; }
	if ( graph_v2.log )		{ dbrecord_desc.fields |= WBO2_FIELD_VU2; dbrecord_item_size += 2; }
}

UInt32 Log_Read_Header( Char *recordp, UInt32 offset, WBO2Record_Header *header )
{
	MemMove( &header->next,			&recordp[ offset ], 2 );	offset += 2;
	MemMove( &header->prev,			&recordp[ offset ], 2 );	offset += 2;
	MemMove( &header->samples,		&recordp[ offset ], 4 );	offset += 4;
	MemMove( &header->data_offset,	&recordp[ offset ], 4 );	offset += 4;

	return offset;
}

UInt32 Log_Write_Header( MemPtr recordp, UInt32 offset, WBO2Record_Header *header )
{
	DmWrite( recordp, offset, &header->next,		2 );	offset += 2;
	DmWrite( recordp, offset, &header->prev,		2 );	offset += 2;
	DmWrite( recordp, offset, &header->samples,		4 );	offset += 4;
	DmWrite( recordp, offset, &header->data_offset,	4 );	offset += 4;

	return offset;
}

UInt32 Log_Read_Desc( Char *recordp, UInt32 offset, WBO2Record_Desc *desc )
{
	MemMove( &desc->fields,		&recordp[ offset ], 2 );		offset += 2;
	MemMove( desc->name,		&recordp[ offset ], 100 + 1 );	offset += 100 + 1;
	MemMove( desc->notes,		&recordp[ offset ], 923 + 1 );	offset += 923 + 1;
	MemMove( desc->v1_label,	&recordp[ offset ], 10 + 1 );	offset += 10 + 1;
	MemMove( desc->v2_label,	&recordp[ offset ], 10 + 1 );	offset += 10 + 1;

	return offset;
}

UInt32 Log_Write_Desc( MemPtr recordp, UInt32 offset, WBO2Record_Desc *desc )
{
	DmWrite( recordp, offset, &desc->fields,	2 );		offset += 2;
	DmWrite( recordp, offset, desc->name,		100 + 1 );	offset += 100 + 1;
	DmWrite( recordp, offset, desc->notes,		923 + 1 );	offset += 923 + 1;
	DmWrite( recordp, offset, desc->v1_label,	10 + 1 );	offset += 10 + 1;
	DmWrite( recordp, offset, desc->v2_label,	10 + 1 );	offset += 10 + 1;

	return offset;
}

void Log_Flush()
{
	UInt16 newrecord_index, dbattr;
	UInt32 offset;
	MemPtr recordp;
	MemHandle dbrecord;

	/*
	 * Flush the data to the database
	 */

	if ( dbrecord_index == -1 )
	{
		dbrecord_index				= DmNumRecords( dbP );
		dbrecord_first_index		= dbrecord_index;
		dbrecord_size				= WBO2_RECORD_HEADER_SIZE + WBO2_RECORD_DESC_SIZE + dbrecord_queue_pos;
		dbrecord					= DmNewRecord( dbP, &dbrecord_index, dbrecord_size );

		/* Put the first record in category 1 */
		DmRecordInfo( dbP, dbrecord_index, &dbattr, NULL, NULL );
		dbattr						&= 0xFFF0;
		dbattr						|= 0x0001;
		DmSetRecordInfo( dbP, dbrecord_index, &dbattr, NULL );

		recordp						= MemHandleLock( dbrecord );

		dbrecord_header.samples		= dbrecord_queue_samples;
		dbrecord_header.data_offset	= WBO2_RECORD_HEADER_SIZE + WBO2_RECORD_DESC_SIZE;
		offset						= Log_Write_Header( recordp,	0, &dbrecord_header );
		offset						= Log_Write_Desc( recordp,		offset,	&dbrecord_desc );

		DmWrite( recordp, offset,	dbrecord_queue,		dbrecord_queue_pos );

		MemPtrUnlock( recordp );
		DmReleaseRecord( dbP, dbrecord_index, 0 );
	}
	else
	{
		if ( dbrecord_size + dbrecord_queue_pos >= WBO2_RECORD_MAX_SIZE )
		{
			/*
			 * Update the "next" field in the current record to point to the new record
			 */

			newrecord_index				= DmNumRecords( dbP );
			dbrecord_header.next		= newrecord_index;

			dbrecord					= DmGetRecord( dbP, dbrecord_index );
			recordp						= MemHandleLock( dbrecord );

			Log_Write_Header( recordp, 0, &dbrecord_header );

			MemPtrUnlock( recordp );
			DmReleaseRecord( dbP, dbrecord_index, 0 );

			/*
			 * Create a new record, with prev pointing to the old record
			 */

			dbrecord_header.next		= -1;
			dbrecord_header.prev		= dbrecord_index;
			dbrecord_header.samples		= dbrecord_queue_samples;
			dbrecord_header.data_offset	= WBO2_RECORD_HEADER_SIZE;

			dbrecord_index				= newrecord_index;
			dbrecord_size				= WBO2_RECORD_HEADER_SIZE + dbrecord_queue_pos;
			dbrecord					= DmNewRecord( dbP, &dbrecord_index, dbrecord_size );

			/* Put the new record in category 2 */
			DmRecordInfo( dbP, dbrecord_index, &dbattr, NULL, NULL );
			dbattr						&= 0xFFF0;
			dbattr						|= 0x0002;
			DmSetRecordInfo( dbP, dbrecord_index, &dbattr, NULL );

			recordp						= MemHandleLock( dbrecord );
			offset						= Log_Write_Header( recordp, 0, &dbrecord_header );

			DmWrite( recordp, offset,	dbrecord_queue,		dbrecord_queue_pos );

			MemPtrUnlock( recordp );
			DmReleaseRecord( dbP, dbrecord_index, 0 );
		}
		else
		{
			/*
			 * Resize the current record and output the queue
			 */

			dbrecord				= DmResizeRecord( dbP, dbrecord_index, dbrecord_size + dbrecord_queue_pos );
			recordp					= MemHandleLock( dbrecord );
			dbrecord_header.samples	+= dbrecord_queue_samples;

			Log_Write_Header( recordp, 0, &dbrecord_header );
			DmWrite( recordp, dbrecord_size, dbrecord_queue, dbrecord_queue_pos );

			MemPtrUnlock( recordp );
			DmReleaseRecord( dbP, dbrecord_index, 0 );

			dbrecord_size			+= dbrecord_queue_pos;
		}
	}

	dbrecord_queue_pos		= 0;
	dbrecord_queue_samples	= 0;
}

void Log_Data( int marker )
{
	int i;

	if ( dbrecord_queue_pos + dbrecord_item_size >= sizeof( dbrecord_queue ) )
	{
		Log_Flush();
	}

	dbrecord_queue_samples++;

	if ( marker )
	{
		for ( i = 0; i < dbrecord_item_size; i++ )
		{
			dbrecord_queue[ dbrecord_queue_pos++ ]	= 0;
		}
	}
	else
	{
		if ( graph_afr.log )
		{
			dbrecord_queue[ dbrecord_queue_pos++ ]	= ( Vwb & 0xFF00 ) >> 8;
			dbrecord_queue[ dbrecord_queue_pos++ ]	= Vwb & 0x00FF;
		}

		if ( graph_rpm.log )
		{
			dbrecord_queue[ dbrecord_queue_pos++ ]	= ( RPM & 0xFF00 ) >> 8;
			dbrecord_queue[ dbrecord_queue_pos++ ]	= RPM & 0x00FF;
		}
		
		if ( graph_v1.log )
		{
			dbrecord_queue[ dbrecord_queue_pos++ ]	= ( Vuser1 & 0xFF00 ) >> 8;
			dbrecord_queue[ dbrecord_queue_pos++ ]	= Vuser1 & 0x00FF;
		}

		if ( graph_v2.log )
		{
			dbrecord_queue[ dbrecord_queue_pos++ ]	= ( Vuser2 & 0xFF00 ) >> 8;
			dbrecord_queue[ dbrecord_queue_pos++ ]	= Vuser2 & 0x00FF;
		}
	}
}

void Log_End()
{
	RectangleType fillrect;

	fillrect.topLeft.x	= 159 - FntCharsWidth( "Logging", 7 );
	fillrect.topLeft.y	= 0;
	fillrect.extent.x	= 160 - fillrect.topLeft.x;
	fillrect.extent.y	= FntCharHeight();

	WinEraseRectangle( &fillrect, 0 );

	logActive = 0;

	Log_Flush();
	FrmPopupForm( formSaveLog );
}

void Log_Delete_Chain( UInt16 dbrecord_index )
{
	UInt16 index;
	MemPtr recordp;
	MemHandle dbrecord;
	WBO2Record_Header header;

	index			= dbrecord_index;
	while ( index != -1 )
	{
		dbrecord	= DmGetRecord( dbP, index );
		recordp		= MemHandleLock( dbrecord );

		Log_Read_Header( recordp, 0, &header );
		MemPtrUnlock( recordp );
		DmReleaseRecord( dbP, index, 0 );
		DmRemoveRecord( dbP, index );

		index		= header.next;
	}
}

void Log_Read_Chain( UInt16 dbrecord_index, WBO2Record_Header **header_chain, WBO2Record_Desc *desc, UInt32 *total_samples )
{
	UInt16 index;
	UInt32 sample_counter;
	MemPtr recordp;
	MemHandle dbrecord;
	WBO2Record_Header *header, *temp;

	header			= MemPtrNew( sizeof( WBO2Record_Header ) );
	*header_chain	= header;
	index			= dbrecord_index;
	sample_counter	= 0;
	header->prevp	= NULL;

	while ( index != -1 )
	{
		header->dbrecord_index	= index;
		header->first_sample	= sample_counter;
		header->nextp			= NULL;

		dbrecord				= DmGetRecord( dbP, index );
		if ( dbrecord == NULL )	SysFatalAlert( "DmGetRecord in Log_Read_Chain failed" );
		recordp					= MemHandleLock( dbrecord );

		Log_Read_Header( recordp, 0, header );
		if ( index == dbrecord_index )
		{
			Log_Read_Desc( recordp, WBO2_RECORD_HEADER_SIZE, desc );
		}

		MemPtrUnlock( recordp );
		DmReleaseRecord( dbP, index, 0 );

		sample_counter			+= header->samples;
		index					= header->next;

		if ( header->next != -1 )
		{
			temp				= MemPtrNew( sizeof( WBO2Record_Header ) );
			header->nextp		= temp;
			temp->prevp			= header;
			header				= temp;
		}
	}

	*total_samples	= sample_counter;
}

void Log_Free_Chain( WBO2Record_Header **header_chain )
{
	WBO2Record_Header *header, *next;

	header		= *header_chain;
	while ( header )
	{
		next	= header->nextp;
		MemPtrFree( header );
		header	= next;
	}
}
