/*
 * FMDLog
 * Wideband O2 Display/Datalogging Application
 * For the TechEdge DIY-WB series of devices
 * (c) 2002-2003, Jonathan Burchmore
 *
 * Your use of this software is governed by the GNU General Public License, v2.
 * See the file COPYING.txt, or http://www.fsf.org/copyleft/gpl.txt
 *
 * This software is provided with ABSOLUTELY NO WARRANTY.  See the GNU General
 * public license for additional information.
 */

#include <PalmOS.h>
#include "resource.h"
#include "fmdlog.h"

/*
 * Scaling Functions
 */

char *scaling_title;
UInt32 *scaling_mv,
		scaling_v1_mv[ SCALING_STEPS ] = { 200,	4800, },
		scaling_v2_mv[ SCALING_STEPS ] = { 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, };
UInt32 *scaling_out,
		scaling_v1_out[ SCALING_STEPS ] = { 20,		250, },
		scaling_v2_out[ SCALING_STEPS ] = { 16, 15, 14, 13, 12, 11, 10, 9, 8, 7, 6, 5, 4, 3, 2, 1, };
int *scaling_points,
	 scaling_v1_points = 2,
	 scaling_v2_points = 16;

UInt32 Sensor_Scale( UInt32 input, UInt32 *mv, UInt32 *out, int points )
{
	int i;

	if ( mv[ 0 ] > input )		return 0;
	
	for ( i = 1; i < points; i++ )
	{
		if ( mv[ i ] >= input )	break;
	}

	if ( i == points )			return 0;

	return ( Int32 ) out[ i ] +
		   ( ( ( Int32 ) out[ i ] - ( Int32 ) out[ i - 1 ] ) * ( ( Int32 ) input - ( Int32 ) mv[ i ] ) ) /
		     ( ( Int32 ) mv[ i ] - ( Int32 ) mv[ i - 1 ] );
}

Boolean ScalingForm_HandleEvent( EventPtr event )
{
	FormPtr frm;
	int points;

	if ( event->eType == frmOpenEvent )
	{
		frm				= FrmGetActiveForm();

		FrmSetTitle( frm, scaling_title );

		if ( *scaling_points >= 1 )		{ Preferences_Set_Number( frm, textScale1mV,	scaling_mv[ 0 ] );	Preferences_Set_Number( frm, textScale1Output,	scaling_out[ 0 ] ); }
		if ( *scaling_points >= 2 )		{ Preferences_Set_Number( frm, textScale2mV,	scaling_mv[ 1 ] );	Preferences_Set_Number( frm, textScale2Output,	scaling_out[ 1 ] ); }
		if ( *scaling_points >= 3 )		{ Preferences_Set_Number( frm, textScale3mV,	scaling_mv[ 2 ] );	Preferences_Set_Number( frm, textScale3Output,	scaling_out[ 2 ] ); }
		if ( *scaling_points >= 4 )		{ Preferences_Set_Number( frm, textScale4mV,	scaling_mv[ 3 ] );	Preferences_Set_Number( frm, textScale4Output,	scaling_out[ 3 ] ); }
		if ( *scaling_points >= 5 )		{ Preferences_Set_Number( frm, textScale5mV,	scaling_mv[ 4 ] );	Preferences_Set_Number( frm, textScale5Output,	scaling_out[ 4 ] ); }
		if ( *scaling_points >= 6 )		{ Preferences_Set_Number( frm, textScale6mV,	scaling_mv[ 5 ] );	Preferences_Set_Number( frm, textScale6Output,	scaling_out[ 5 ] ); }
		if ( *scaling_points >= 7 )		{ Preferences_Set_Number( frm, textScale7mV,	scaling_mv[ 6 ] );	Preferences_Set_Number( frm, textScale7Output,	scaling_out[ 6 ] ); }
		if ( *scaling_points >= 8 )		{ Preferences_Set_Number( frm, textScale8mV,	scaling_mv[ 7 ] );	Preferences_Set_Number( frm, textScale8Output,	scaling_out[ 7 ] ); }
		if ( *scaling_points >= 9 )		{ Preferences_Set_Number( frm, textScale9mV,	scaling_mv[ 8 ] );	Preferences_Set_Number( frm, textScale9Output,	scaling_out[ 8 ] ); }
		if ( *scaling_points >= 10 )	{ Preferences_Set_Number( frm, textScale10mV,	scaling_mv[ 9 ] );	Preferences_Set_Number( frm, textScale10Output,	scaling_out[ 9 ] ); }
		if ( *scaling_points >= 11 )	{ Preferences_Set_Number( frm, textScale11mV,	scaling_mv[ 10 ] );	Preferences_Set_Number( frm, textScale11Output,	scaling_out[ 10 ] ); }
		if ( *scaling_points >= 12 )	{ Preferences_Set_Number( frm, textScale12mV,	scaling_mv[ 11 ] );	Preferences_Set_Number( frm, textScale12Output,	scaling_out[ 11 ] ); }
		if ( *scaling_points >= 13 )	{ Preferences_Set_Number( frm, textScale13mV,	scaling_mv[ 12 ] );	Preferences_Set_Number( frm, textScale13Output,	scaling_out[ 12 ] ); }
		if ( *scaling_points >= 14 )	{ Preferences_Set_Number( frm, textScale14mV,	scaling_mv[ 13 ] );	Preferences_Set_Number( frm, textScale14Output,	scaling_out[ 13 ] ); }
		if ( *scaling_points >= 15 )	{ Preferences_Set_Number( frm, textScale15mV,	scaling_mv[ 14 ] );	Preferences_Set_Number( frm, textScale15Output,	scaling_out[ 14 ] ); }
		if ( *scaling_points >= 16 )	{ Preferences_Set_Number( frm, textScale16mV,	scaling_mv[ 15 ] );	Preferences_Set_Number( frm, textScale16Output,	scaling_out[ 15 ] ); }

		FrmDrawForm( frm );
		return true;
	}
	else if ( event->eType == ctlSelectEvent )
	{
		if ( event->data.ctlSelect.controlID == buttonScaleSave )
		{
			frm				= FrmGetActiveForm();
			points			= 0;

			Preferences_Get_NumberU32( frm, textScale1mV,	&scaling_mv[ points ] );	Preferences_Get_NumberU32( frm, textScale1Output,	&scaling_out[ points ] );	if ( scaling_mv[ points ] && scaling_out[ points ] )	( points )++;
			Preferences_Get_NumberU32( frm, textScale2mV,	&scaling_mv[ points ] );	Preferences_Get_NumberU32( frm, textScale2Output,	&scaling_out[ points ] );	if ( scaling_mv[ points ] && scaling_out[ points ] )	( points )++;
			Preferences_Get_NumberU32( frm, textScale3mV,	&scaling_mv[ points ] );	Preferences_Get_NumberU32( frm, textScale3Output,	&scaling_out[ points ] );	if ( scaling_mv[ points ] && scaling_out[ points ] )	( points )++;
			Preferences_Get_NumberU32( frm, textScale4mV,	&scaling_mv[ points ] );	Preferences_Get_NumberU32( frm, textScale4Output,	&scaling_out[ points ] );	if ( scaling_mv[ points ] && scaling_out[ points ] )	( points )++;
			Preferences_Get_NumberU32( frm, textScale5mV,	&scaling_mv[ points ] );	Preferences_Get_NumberU32( frm, textScale5Output,	&scaling_out[ points ] );	if ( scaling_mv[ points ] && scaling_out[ points ] )	( points )++;
			Preferences_Get_NumberU32( frm, textScale6mV,	&scaling_mv[ points ] );	Preferences_Get_NumberU32( frm, textScale6Output,	&scaling_out[ points ] );	if ( scaling_mv[ points ] && scaling_out[ points ] )	( points )++;
			Preferences_Get_NumberU32( frm, textScale7mV,	&scaling_mv[ points ] );	Preferences_Get_NumberU32( frm, textScale7Output,	&scaling_out[ points ] );	if ( scaling_mv[ points ] && scaling_out[ points ] )	( points )++;
			Preferences_Get_NumberU32( frm, textScale8mV,	&scaling_mv[ points ] );	Preferences_Get_NumberU32( frm, textScale8Output,	&scaling_out[ points ] );	if ( scaling_mv[ points ] && scaling_out[ points ] )	( points )++;
			Preferences_Get_NumberU32( frm, textScale9mV,	&scaling_mv[ points ] );	Preferences_Get_NumberU32( frm, textScale9Output,	&scaling_out[ points ] );	if ( scaling_mv[ points ] && scaling_out[ points ] )	( points )++;
			Preferences_Get_NumberU32( frm, textScale10mV,	&scaling_mv[ points ] );	Preferences_Get_NumberU32( frm, textScale10Output,	&scaling_out[ points ] );	if ( scaling_mv[ points ] && scaling_out[ points ] )	( points )++;
			Preferences_Get_NumberU32( frm, textScale11mV,	&scaling_mv[ points ] );	Preferences_Get_NumberU32( frm, textScale11Output,	&scaling_out[ points ] );	if ( scaling_mv[ points ] && scaling_out[ points ] )	( points )++;
			Preferences_Get_NumberU32( frm, textScale12mV,	&scaling_mv[ points ] );	Preferences_Get_NumberU32( frm, textScale12Output,	&scaling_out[ points ] );	if ( scaling_mv[ points ] && scaling_out[ points ] )	( points )++;
			Preferences_Get_NumberU32( frm, textScale13mV,	&scaling_mv[ points ] );	Preferences_Get_NumberU32( frm, textScale13Output,	&scaling_out[ points ] );	if ( scaling_mv[ points ] && scaling_out[ points ] )	( points )++;
			Preferences_Get_NumberU32( frm, textScale14mV,	&scaling_mv[ points ] );	Preferences_Get_NumberU32( frm, textScale14Output,	&scaling_out[ points ] );	if ( scaling_mv[ points ] && scaling_out[ points ] )	( points )++;
			Preferences_Get_NumberU32( frm, textScale15mV,	&scaling_mv[ points ] );	Preferences_Get_NumberU32( frm, textScale15Output,	&scaling_out[ points ] );	if ( scaling_mv[ points ] && scaling_out[ points ] )	( points )++;
			Preferences_Get_NumberU32( frm, textScale16mV,	&scaling_mv[ points ] );	Preferences_Get_NumberU32( frm, textScale16Output,	&scaling_out[ points ] );	if ( scaling_mv[ points ] && scaling_out[ points ] )	( points )++;

			*scaling_points	= points;

			Config_Save();
			FrmReturnToForm( 0 );
			return true;
		}
		else if ( event->data.ctlSelect.controlID == buttonScaleCancel )
		{
			FrmReturnToForm( 0 );
			return true;
		}
	}

	return false;
}

